/*
 * This program creates a vehicle Object.
 *
 * Author:
 *     Phil List, 2014-2015
 *		Brendan Bello, 2014-2015
 */

// Global variables

var canvas;                 // holds the HTML canvas
var context;                // holds the context of the HTML canvas
var UIDs = [];				// array to capture all setInterval UIDs
var drawCount = 0;			// number of times "Draw" was pressed
var trackWidth = 15;		// space between tracks on which vehicles move horizontally
var startLine = 35;         // the x-position of the starting line
var v1, v2, v3, v4, v5, v6;     // the vehicles that will be defined soon
var firstOffLine = null;    // this will track which vehicle object is told to "go" first
var FPS = 20;               // Frames Per Second (how often the vehicles per sec are redrawn)

// This is the function that creates Vehicle objects
Vehicle = function(myMake, myModel, myYear, myColor, mySize, mySpeed, myTrack) {
    this.make = myMake;
    this.model = myModel;
    this.year = myYear;
    this.color = myColor;
    this.len = mySize;
    this.speed = Math.floor(mySpeed/10);
    this.track = myTrack;
    this.x = 0;
    this.intervalId = 0;
    this.finished = false;
    
    this.move = function (dist) {
        if(typeof dist === "undefined"){
            this.x += this.speed;
        }else{
            this.x += dist;
        }
        if(this.x > canvas.width && !this.finished){
            this.finished = true;
            console.log("The "+this.make+" "+this.model+" just finished!");
            clearInterval(this.intervalId);       // stop drawing vehicle off the canvas
            document.getElementById("news").innerHTML = "The "+this.make+" "+this.model+" just crossed the finish line!";
        }
    };
    
    this.draw = function () {
        context.fillStyle="white";
        context.fillRect(startLine - this.len + this.x - this.speed, 5 + this.track*trackWidth, this.len, this.len - 4);
    	context.fillStyle=this.color;
    	context.fillRect(startLine - this.len + this.x, 5 + this.track*trackWidth, this.len, this.len - 4);
    };
    
    this.start = function () {
        var sr = this;
        this.intervalId = setInterval(function(){animate(sr);}, Math.floor(1000/FPS));

        UIDs[drawCount++] = this.intervalId;

        if(firstOffLine == null){
            firstOffLine = this.intervalId;
        }
    }

    
}

init = function() {

    // Store the HTML canvas and its context in variables.
    canvas = document.getElementById("canvas");
    context = canvas.getContext("2d");

    // Draw the vehicle on the screen (make, model, year, color, len, top speed (in mph), track)
	v1 = new Vehicle("Ford", "Focus", 2007, "orange", 11, 110, 1);
	v2 = new Vehicle("Jeep", "Cherokee", 1995, "brown", 16, 85, 2);
	v3 = new Vehicle("home-built", "moped", 2009, "purple", 6, 40, 3);
    v4 = new Vehicle("Semi", "truck", 1980, "grey", 26, 65, 4);
    v5 = new Vehicle("Ferrari", "Maranello", 2000, "red", 12, 200, 6);
    v6 = new Vehicle("Chevy","Corvette", 2015, "black", 13, 140, 8);

	drawStartLine(startLine);
	v1.draw();
	v2.draw();
    v3.draw();
    v4.draw();
    v5.draw();
    v6.draw();
}


// Each vehicle calls this function every timestep
animate = function(self){
    // We only want to clear the canvas once per timestep,
    // so only the vehicle that started driving first has the
    // canvas cleared during its call of this function.
    if(self.intervalId == firstOffLine){
        context.clearRect(0, 0, canvas.width, canvas.height);
        drawStartLine(startLine);
    }

    // We want to calculate & draw each vehicle each timestep
    self.move();
    self.draw();
}

// Each car starts driving
startRace = function(){
    if(firstOffLine != null){
        return;
    }
    v1.start();
    v2.start();
    v3.start();
    v4.start();
    v5.start();
    v6.start();
}

// draws the starting line
drawStartLine = function(xValue){
	context.beginPath();
	context.moveTo(xValue, 10);
    context.lineTo(xValue, canvas.height - 10);
    context.stroke();
}

// stop every vehicle by stopping every instance of 'setInterval'
stopTheMadness = function(mvBack){
	for(var i = 0; i < drawCount; i++){
		clearInterval(UIDs[i]);
	}
    firstOffLine = null;
}